@@ -211,7 +211,7 @@ static METHOD Regex__s(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD Class__s(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_write_ltype(ctx, sfp[1].w, knh_Object_cid(sfp[0].o));
+	knh_write_ltype(ctx, sfp[1].w, knh_Class_cid(sfp[0].c));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -385,13 +385,12 @@ static METHOD Tuple__k(Ctx *ctx, knh_sfp_t *sfp)
 		knh_format(ctx, w, METHODN__k, o->third, KNH_NULL);
 	}
 	else {
-		size_t i, c = 0;
-		for(i = 0; o->size; i++) {
-			if(c > 0) {
+		size_t i;
+		for(i = 0; i < o->size; i++) {
+			if(i > 0) {
 				knh_write_delim(ctx,w);
 			}
-			knh_format(ctx, w, METHODN__k, o->list[c], KNH_NULL);
-			c++;
+			knh_format(ctx, w, METHODN__k, o->list[i], KNH_NULL);
 		}
 	}
 	knh_putc(ctx, w, ')');
