@@ -344,12 +344,14 @@ static METHOD String_replace(Ctx *ctx, knh_sfp_t *sfp)
 	knh_bytes_t target = __tobytes(sfp[1].s);
 	knh_bytes_t alt = __tobytes(sfp[2].s);
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
-	int search_flag= 0, ch = target.buf[0], i;
+	int search_flag= 0, ch = target.buf[0];
+	size_t i;
 
-	if (base.len == 0 || target.len == 0) KNH_RETURN(ctx, sfp, sfp[0].o);
-	for(i = 0; i < base.len - target.len+1; i++) {
+	if (base.len == 0 || target.len == 0 || base.len < target.len)
+		KNH_RETURN(ctx, sfp, sfp[0].o);
+	for(i = 0; i < (base.len - target.len+1); i++) {
 		if(base.buf[i] == ch && knh_bytes_equals_(base, i, target)) {
-		    knh_Bytes_write(ctx, cwb->ba, alt);
+			knh_Bytes_write(ctx, cwb->ba, alt);
 			i += target.len - 1;
 			search_flag = 1;
 		}else {
@@ -506,7 +508,12 @@ static METHOD String_split(Ctx *ctx, knh_sfp_t *sfp)
 				else {
 					knh_bytes_t t = knh_bytes_first(base, loc);
 					if(istrim) t = knh_bytes_trim(t);
-					knh_Array_add(ctx, a, UP(new_String(ctx, t, sfp[0].s)));
+					// FIXME
+					// We want to use sfp[0].s in order to save memory.
+					// knh_Array_add(ctx, a, UP(new_String(ctx, t, sfp[0].s)));
+					// But, in some case that use raw string (t.buf), it does not work.
+					// "hello world".split(" ") -> t = {"hello world", 5}
+					knh_Array_add(ctx, a, UP(new_String(ctx, t, NULL)));
 				}
 				base.buf = base.buf + loc + delim.len;
 				base.len = base.len - loc - delim.len;
