@@ -37,7 +37,7 @@ extern "C" {
 /* ======================================================================== */
 /* [method] */
 
-KNHAPI(void) knh_putc(Ctx *ctx, OutputStream *w, int ch)
+KNHAPI(void) knh_putc(Ctx *ctx, knh_OutputStream_t *w, int ch)
 {
 	DBG2_ASSERT(IS_OutputStream(w));
 	knh_OutputStream_putc(ctx, w, ch);
@@ -45,7 +45,7 @@ KNHAPI(void) knh_putc(Ctx *ctx, OutputStream *w, int ch)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_write(Ctx *ctx, OutputStream *w, knh_bytes_t s)
+KNHAPI(void) knh_write(Ctx *ctx, knh_OutputStream_t *w, knh_bytes_t s)
 {
 	DBG2_ASSERT(IS_OutputStream(w));
 	knh_OutputStream_write(ctx, w, s);
@@ -53,7 +53,7 @@ KNHAPI(void) knh_write(Ctx *ctx, OutputStream *w, knh_bytes_t s)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_flush(Ctx *ctx, OutputStream *w)
+KNHAPI(void) knh_flush(Ctx *ctx, knh_OutputStream_t *w)
 {
 	DBG2_ASSERT(IS_OutputStream(w));
 	knh_OutputStream_flush(ctx, w);
@@ -61,7 +61,7 @@ KNHAPI(void) knh_flush(Ctx *ctx, OutputStream *w)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_print(Ctx *ctx, OutputStream *w, knh_bytes_t s)
+KNHAPI(void) knh_print(Ctx *ctx, knh_OutputStream_t *w, knh_bytes_t s)
 {
 	DBG2_ASSERT(IS_OutputStream(w));
 	knh_OutputStream_print_(ctx, w, s, 0);
@@ -69,7 +69,7 @@ KNHAPI(void) knh_print(Ctx *ctx, OutputStream *w, knh_bytes_t s)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_println(Ctx *ctx, OutputStream *w, knh_bytes_t s)
+KNHAPI(void) knh_println(Ctx *ctx, knh_OutputStream_t *w, knh_bytes_t s)
 {
 	DBG2_ASSERT(IS_OutputStream(w));
 	knh_OutputStream_print_(ctx, w, s, 1);
@@ -77,10 +77,9 @@ KNHAPI(void) knh_println(Ctx *ctx, OutputStream *w, knh_bytes_t s)
 
 /* ======================================================================== */
 
-KNHAPI(void) knh_write_EOL(Ctx *ctx, OutputStream *w)
+KNHAPI(void) knh_write_EOL(Ctx *ctx, knh_OutputStream_t *w)
 {
-	DBG2_ASSERT(IS_OutputStream(w));
-	knh_OutputStream_write(ctx, w, knh_String_tobytes(DP(w)->NEWLINE));
+	knh_OutputStream_write(ctx, w, __tobytes(DP(w)->NEWLINE));
 	if(knh_OutputStream_isAutoFlush(w)) {
 		knh_OutputStream_flush(ctx, w);
 	}
@@ -89,35 +88,61 @@ KNHAPI(void) knh_write_EOL(Ctx *ctx, OutputStream *w)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_write_TAB(Ctx *ctx, OutputStream *w)
+KNHAPI(void) knh_write_TAB(Ctx *ctx, knh_OutputStream_t *w)
 {
-	DBG2_ASSERT(IS_OutputStream(w));
-	knh_OutputStream_write(ctx, w, knh_String_tobytes(DP(w)->TAB));
+	knh_OutputStream_write(ctx, w, __tobytes(DP(w)->TAB));
 }
 
-#define _knh_write_delim(ctx, w)    knh_write(ctx, w, STEXT(", "))
-#define _knh_write_dots(ctx, w)     knh_write(ctx, w, STEXT("..."))
-
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_write_BOL(Ctx *ctx, OutputStream *w)
+KNHAPI(void) knh_write_BOL(Ctx *ctx, knh_OutputStream_t *w)
 {
 	knh_intptr_t i, n = DP(w)->indent;
-	for(i = 0; i < n; i++) knh_write_TAB(ctx, w);
+	if(!knh_OutputStream_isBOL(w)) {
+		knh_write_EOL(ctx, w);
+	}
+	for(i = 0; i < n; i++) {
+		//knh_write(ctx, w, STEXT("/*"));
+		//knh_write_ifmt(ctx, w, KNH_INT_FMT, i);
+		//knh_write(ctx, w, STEXT("*/"));
+		knh_OutputStream_write(ctx, w, __tobytes(DP(w)->TAB));
+	}
 	knh_OutputStream_setBOL(w, 0);
 }
 
+/* ------------------------------------------------------------------------ */
+
+void knh_write_begin(Ctx *ctx, knh_OutputStream_t *w, int ch)
+{
+	if(ch != 0) {
+		knh_putc(ctx, w, ch);
+		knh_write_EOL(ctx, w);
+	}
+	DP(w)->indent++;
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_write_end(Ctx *ctx, knh_OutputStream_t *w, int ch)
+{
+	DP(w)->indent--;
+	if(ch != 0) {
+		knh_write_BOL(ctx, w);
+		knh_putc(ctx, w, ch);
+	}
+}
+
 /* ======================================================================== */
 /* [datatype] */
 
-void knh_write_char(Ctx *ctx, OutputStream *w, char *s)
+void knh_write_char(Ctx *ctx, knh_OutputStream_t *w, char *s)
 {
 	knh_write(ctx, w, B(s));
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write__p(Ctx *ctx, OutputStream *w, void *ptr)
+void knh_write__p(Ctx *ctx, knh_OutputStream_t *w, void *ptr)
 {
 	char buf[KNH_INT_FMTSIZ];
 	knh_snprintf(buf, sizeof(buf), "%p", ptr);
@@ -126,7 +151,7 @@ void knh_write__p(Ctx *ctx, OutputStream *w, void *ptr)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_dfmt(Ctx *ctx, OutputStream *w, char *fmt, knh_intptr_t n)
+void knh_write_dfmt(Ctx *ctx, knh_OutputStream_t *w, char *fmt, knh_intptr_t n)
 {
 	char buf[KNH_INT_FMTSIZ];
 	knh_snprintf(buf, sizeof(buf), fmt, n);
@@ -135,33 +160,26 @@ void knh_write_dfmt(Ctx *ctx, OutputStream *w, char *fmt, knh_intptr_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_ifmt(Ctx *ctx, OutputStream *w, char *fmt, knh_int_t n)
+void knh_write_ifmt(Ctx *ctx, knh_OutputStream_t *w, char *fmt, knh_int_t n)
 {
 	char buf[KNH_INT_FMTSIZ];
 	knh_snprintf(buf, sizeof(buf), fmt, n);
 	knh_write(ctx, w, B(buf));
 }
 
-//#define _knh_write__i(ctx, w, n)   knh_write_dfmt(ctx, w, KNH_INTPTR_FMT, n)
-//#define _knh_write__u(ctx, w, n)   knh_write_dfmt(ctx, w, KNH_UINT_FMT, n)
-//#define _knh_write__x(ctx, w, n)   knh_write_dfmt(ctx, w, KNH_INTPTR_FMTX, n)
-
 /* ------------------------------------------------------------------------ */
 
-void knh_write_ffmt(Ctx *ctx, OutputStream *w, char *fmt, knh_float_t n)
+void knh_write_ffmt(Ctx *ctx, knh_OutputStream_t *w, char *fmt, knh_float_t n)
 {
 	char buf[KNH_FLOAT_FMTSIZ];
 	knh_snprintf(buf, sizeof(buf), fmt, n);
 	knh_write(ctx, w, B(buf));
 }
 
-//#define _knh_write__f(ctx, w, f)  knh_write_ffmt(ctx, w, KNH_FLOAT_FMT, f)
-//#define _knh_write__e(ctx, w, f)  knh_write_ffmt(ctx, w, KNH_FLOAT_FMTE, f)
-
 /* ------------------------------------------------------------------------ */
 /* [flag] */
 
-void knh_write__flag(Ctx *ctx, OutputStream *w, knh_flag_t flag)
+void knh_write__flag(Ctx *ctx, knh_OutputStream_t *w, knh_flag_t flag)
 {
 	knh_intptr_t i;
 	knh_flag_t f = 1 << 15;
@@ -180,23 +198,34 @@ void knh_write__flag(Ctx *ctx, OutputStream *w, knh_flag_t flag)
 /* language */
 /* ------------------------------------------------------------------------ */
 
-void knh_write_cid(Ctx *ctx, OutputStream *w, knh_class_t cid)
+void knh_write_cid(Ctx *ctx, knh_OutputStream_t *w, knh_class_t cid)
 {
 	knh_write(ctx, w, B(CTXCLASSN(cid)));
 }
 
 /* ------------------------------------------------------------------------ */
 
-#define _knh_write_fn(ctx, w, fn)   knh_write(ctx, w, B(FIELDN(fn)))
+void knh_write_lcid(Ctx *ctx, knh_OutputStream_t *w, knh_class_t cid)
+{
+	DBG2_ASSERT_cid(cid);
+	knh_write(ctx, w, __tobytes(ClassTable(cid).lname));
+}
 
-void knh_write_mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn)
+/* ------------------------------------------------------------------------ */
+
+void knh_write_mn(Ctx *ctx, knh_OutputStream_t *w, knh_type_t rtype, knh_methodn_t mn)
 {
 	char *name = FIELDN(METHODN_TOFIELDN(mn));
 	if(METHODN_IS_MOVTEXT(mn)) {
 		knh_putc(ctx, w, '%');
 	}
 	else if(METHODN_IS_GETTER(mn)) {
-		knh_write(ctx, w, STEXT("get"));
+		if(rtype == TYPE_Boolean) {
+			knh_write(ctx, w, STEXT("is"));
+		}
+		else {
+			knh_write(ctx, w, STEXT("get"));
+		}
 		if(islower(name[0])) {
 			knh_putc(ctx, w, toupper(name[0])); name++;
 		}
@@ -212,30 +241,33 @@ void knh_write_mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_type(Ctx *ctx, OutputStream *w, knh_type_t type)
+void knh_write_type_(Ctx *ctx, knh_OutputStream_t *w, knh_type_t type, int isLongName)
 {
+	knh_class_t cid = CLASS_type(type);
+	DBG2_ASSERT_cid(cid);
 	if(type == TYPE_void) {
 		knh_write(ctx, w, STEXT("void"));
-		return;
 	}
-
-	knh_class_t cid = CLASS_type(type);
-	KNH_ASSERT_cid(cid);
-	char *cname = CTXCLASSN(cid);
-	if(IS_ubxtype(type)) {
-		knh_putc(ctx, w, tolower(cname[0]));
-		knh_write_char(ctx, w, cname + 1);
-		return;
+	else if(type == TYPE_var) {
+		knh_write(ctx, w, STEXT("var"));
 	}
-	knh_write_char(ctx, w, cname);
-	if(!IS_NNTYPE(type)) {
-		knh_putc(ctx, w, '?');
+	else {
+		char *cname = (isLongName) ? knh_ClassTable_CLASSN(ctx, cid) : CTXCLASSN(cid);
+		if(IS_ubxtype(type)) {
+			knh_putc(ctx, w, tolower(cname[0]));
+			knh_write_char(ctx, w, cname + 1);
+			return;
+		}
+		knh_write_char(ctx, w, cname);
+		if(IS_NATYPE(type)) {
+			knh_putc(ctx, w, '?');
+		}
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_fline(Ctx *ctx, OutputStream *w, char *file, int line)
+void knh_write_fline(Ctx *ctx, knh_OutputStream_t *w, char *file, int line)
 {
 	if(line == 0) {
 		knh_write(ctx, w, STEXT("(unknown:0)"));
@@ -252,14 +284,15 @@ void knh_write_fline(Ctx *ctx, OutputStream *w, char *file, int line)
 /* ======================================================================== */
 /* [String] */
 
-#define _knh_write__O(ctx, w, o)    knh_format(ctx, w, METHODN__k, o, KNH_NULL)
-
-KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Object *o, Any *m)
+KNHAPI(void) knh_format(Ctx *ctx, knh_OutputStream_t *w, knh_methodn_t mn, Object *o, Any *m)
 {
-	knh_sfp_t *esp = KNH_LOCAL(ctx);
-	KNH_SETv(ctx, esp[1].o, o);
-	esp[1].data = knh_Object_data(o);
-	knh_esp1_format(ctx, mn, w, m);
+	knh_sfp_t sfpbuf;
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	sfpbuf.o = o;
+	sfpbuf.data = knh_Object_data(o);
+	knh_stack_w(ctx, lsfp, &sfpbuf, mn, w, m);
+	KNH_LOCALBACK(ctx, lsfp);
+
 }
 
 /* ======================================================================== */
@@ -267,7 +300,7 @@ KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Object *o,
 
 static char* knh_vprintf_parseindex(char *p, int *index)
 {
-    char *ptr = p+1;
+	char *ptr = p+1;
 	if(ptr[0] == '{' && isdigit(ptr[1]) && ptr[2] == '}') {
 		*index = ptr[1] - '0';
 		ptr += 3;
@@ -313,7 +346,7 @@ typedef struct {
 
 /* ------------------------------------------------------------------------ */
 
-void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
+void knh_vprintf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, va_list ap)
 {
 	knh_valist_t args[10];
 	char *c = fmt;
@@ -323,8 +356,13 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 	while((ch = *c) != '\0') {
 		c++;
 		if(ch == '%') {
+			int index;
 			ch = *c;
-			int index = bindex++;
+			if(ch == '%') {
+				c++;
+				continue;
+			}
+			index = bindex++;
 			c = knh_vprintf_parseindex(c++, &index);
 			//DBG2_P("bindex=%d, index=%d", bindex, index);
 			switch(ch) {
@@ -362,6 +400,12 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 				case 'B':
 					args[index].atype = VA_BYTES;
 					break;
+				// TODO
+				// we should care if "fmt" has "%%".
+				// sometimes, next args is NULL.
+				case '%':
+					index--;
+					c++;
 				default:
 					bindex--;
 			}
@@ -448,6 +492,13 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 				  knh_print(ctx, w, b);
 				}
 				ch = *c;
+				if(ch == '%') {
+					knh_putc(ctx, w, '%');
+					c++;
+					b.buf = (knh_uchar_t*)c;
+					b.len = 0;
+					continue;
+				}
 				int index = bindex++;
 				c = knh_vprintf_parseindex(++c, &index);
 
@@ -495,7 +546,7 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 						break;
 					case 'M':
 						KNH_ASSERT(args[index].atype == VA_METHODN);
-						knh_write_mn(ctx, w, args[index].mn);
+						knh_write_mn(ctx, w, TYPE_void, args[index].mn);
 						break;
 					case 'C':
 						KNH_ASSERT(args[index].atype == VA_CLASS);
@@ -509,6 +560,9 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 						KNH_ASSERT(args[index].atype == VA_BYTES);
 						knh_write(ctx,w, args[index].bvalue);
 						break;
+					case '%':
+						index--;
+						bindex--;
 					default:
 						//knh_putc(ctx, w, '%');
 						knh_putc(ctx, w, ch);
@@ -532,8 +586,7 @@ void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap)
 
 /* ------------------------------------------------------------------------ */
 
-
-KNHAPI(void) knh_printf(Ctx *ctx, OutputStream *w, char *fmt, ...)
+KNHAPI(void) knh_printf(Ctx *ctx, knh_OutputStream_t *w, char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap , fmt);
@@ -633,7 +686,7 @@ KNHAPI(void) knh_says(Ctx *ctx, int type, char *fmt, ...)
 	}
 	else {
 		va_list args;
-		OutputStream *w = KNH_STDERR;
+		knh_OutputStream_t *w = KNH_STDERR;
 		knh_write(ctx, w, STEXT("[konoha] "));
 		va_start(args, fmt);
 		knh_vprintf(ctx, w, fmt, args);
@@ -645,30 +698,17 @@ KNHAPI(void) knh_says(Ctx *ctx, int type, char *fmt, ...)
 
 /* ------------------------------------------------------------------------ */
 
-typedef void (*f_mt)(Ctx *, Object *, OutputStream *, Any *);
+typedef void (*f_mt)(Ctx *, Object *, knh_OutputStream_t *, Any *);
 
 /* ------------------------------------------------------------------------ */
 
-METHOD knh_fmethod_movableText(Ctx *ctx, knh_sfp_t *sfp)
+METHOD knh_fmethod_formatter(Ctx *ctx, knh_sfp_t *sfp)
 {
 	f_mt f = (f_mt)DP(sfp[-1].mtd)->code;
-	f(ctx, sfp[0].o, (OutputStream*)sfp[1].o, sfp[2].o);
+	f(ctx, sfp[0].o, (knh_OutputStream_t*)sfp[1].o, sfp[2].o);
 	KNH_RETURN_void(ctx, sfp);
 }
 
-/* ------------------------------------------------------------------------ */
-
-knh_bool_t knh_stack_isRecuriveFormatting(Ctx *ctx, knh_sfp_t *sfp, Object *o, OutputStream *w, void *mtdcode)
-{
-	knh_sfp_t *sp = sfp - 1;
-	while(ctx->stack < sp) {
-		if(sp[0].o == o && sp[1].w == w && IS_Method(sp[-1].mtd)) {
-			if(DP(sp[-1].mtd)->code == mtdcode) return 1;
-		}
-		sp--;
-	}
-	return 0;
-}
 
 /* ------------------------------------------------------------------------ */
 
