@@ -947,6 +947,9 @@ void knh_showWelcome(Ctx *ctx, knh_OutputStream_t *w)
 #endif
 #ifdef KNH_USING_RCGC
 	knh_write_char(ctx, w, " refc");
+#endif
+#ifdef KNH_USING_MARKGC
+	knh_write_char(ctx, w, " m&s");
 #endif
 	knh_write_USING_SQLITE3(ctx, w);
 #ifdef KNH_USING_THREAD
